#!/usr/bin/env python

import hashlib
import os
import os.path


_this_dir = os.path.dirname(__file__)


def write_problemlist(problem_dir=None, blacklist=(), symlink=False):
    # remove evidence from previous runs
    for name in os.listdir(_this_dir):
        if os.path.islink(name):
            os.unlink(name)

    # generate new symlinks and problemlist.tex
    f = open(os.path.join(_this_dir, 'problemlist.tex'), 'w')
    if symlink:
        i = 1
    for source in sorted(os.listdir(problem_dir)):
        if source in blacklist:
            continue
        source_dir = os.path.join(problem_dir, source)
        if not os.path.isdir(source_dir):
            continue
        f.write('\\section{%s}\n' % source.replace('_', ' '))
        for problem in sorted(os.listdir(source_dir)):
            path = os.path.join(source_dir, problem)
            if os.path.isfile(path):
                force_link = False
                relpath = os.path.relpath(path, _this_dir)
                if path.endswith('.tex'):
                    if symlink:
                        pname = 'problem%d.tex' % i
                        i += 1
                    else:
                        pname = relpath
                    f.write('\\input{%s}\n' % pname)
                else:  # utility file (e.g. image)
                    pname = '.'.join(problem.split('.')[-2:])
                    if not symlink:
                        force_link = True
                if symlink or force_link:
                    if os.path.islink(pname):
                        ha = hashlib.sha1(open(relpath, 'r').read()).digest()
                        hb = hashlib.sha1(open(pname, 'r').read()).digest()
                        assert ha == hb, (relpath, pname)
                    else:
                        os.symlink(relpath, pname)
    f.close()


if __name__ == '__main__':
    write_problemlist(
        problem_dir=os.path.join(_this_dir, '..', 'problems'),
        blacklist=[
#            'Giancoli_6',
            'Serway_and_Jewett_4',
#            'Serway_and_Jewett_4_venkat',
#            'Serway_and_Jewett_4_wking',
#            'Serway_and_Jewett_8',
#            'Thornton_and_Rex_3_wking',
#            'Young_and_Freedman_12',
#            'wking',
            ]
        )
